import 'dart:async';

import 'package:flutter/material.dart';

class FlexiTimerController with ChangeNotifier {
  Timer? _timer;
  int _counter = 0;
  int get counter => _counter;

  bool _isTimesUp = false;
  bool get isTimesUp => _isTimesUp;
  set isTimesUp(bool v) {
    _isTimesUp = v;
    notifyListeners();
  }

  bool _isRunning = false;
  bool get isTimerRunning => _isRunning;

  void startCountdown(Duration duration) async {
    _counter = duration.inSeconds;
    // Cancel old active timer
    if (_timer != null && _timer!.isActive) {
      _timer!.cancel();
    }
    _isRunning = true;
    _isTimesUp = false;

    _timer = startTimer();
    notifyListeners();
  }

  void stop() async {
    cancelTimer();
    _isTimesUp = false;
    notifyListeners();
  }

  Timer startTimer() =>
      Timer.periodic(const Duration(seconds: 1), (Timer timer) {
        if (_counter > 0) {
          _counter--;
        } else {
          cancelTimer();
          _isTimesUp = true;
        }
        notifyListeners();
      });

  void cancelTimer() async {
    _counter = 0;
    _timer?.cancel();
    _timer = null;
    _isRunning = false;
  }

  @override
  void dispose() {
    _timer?.cancel();
    super.dispose();
  }
}
