import 'package:flexi_timer/flexi_timer.dart';
import 'package:flexi_timer/src/enums/icon_position.dart';
import 'package:flutter/material.dart';

class FlexiTimer extends StatefulWidget {
  final VoidCallback? onComplete;
  final TextStyle? textStyle;
  final Enum timerFormat;
  final Icon? icon;
  final double iconPadding;
  final double? iconSize;
  final Color? iconColor;
  final bool isDiplayIcon;
  final IconPosition iconPosition;
  final FlexiTimerController controller;
  final Duration? duration;

  const FlexiTimer({
    Key? key,
    this.onComplete,
    this.textStyle,
    this.timerFormat = TimerTextFormat.ss,
    this.icon,
    this.iconPadding = 10,
    this.iconSize,
    this.iconColor,
    this.isDiplayIcon = false,
    this.iconPosition = IconPosition.suffix,
    this.duration,
    required this.controller,
  }) : super(key: key);

  @override
  _FlexiTimer createState() => _FlexiTimer();
}

class _FlexiTimer extends State<FlexiTimer> {
  @override
  void initState() {
    super.initState();

    if (widget.duration != null) {
      WidgetsBinding.instance!.addPostFrameCallback(
          (_) => widget.controller.startCountdown(widget.duration!));
    }

    widget.controller
      ..addListener(_setStateListener)
      ..addListener(_onTimeOutListener);
  }

  void _setStateListener() {
    setState(() {});
  }

  void _onTimeOutListener() {
    if (widget.controller.counter == 0 &&
        widget.controller.isTimesUp &&
        widget.onComplete != null) {
      WidgetsBinding.instance?.addPostFrameCallback((_) {
        widget.onComplete!();
      });
    }
  }

  @override
  void dispose() {
    widget.controller
      ..removeListener(_setStateListener)
      ..removeListener(_onTimeOutListener);
    super.dispose();
  }

  @override
  Widget build(BuildContext buildContext) {
    return Row(
      mainAxisAlignment: MainAxisAlignment.center,
      children: [
        widget.iconPosition == IconPosition.prefix && widget.isDiplayIcon
            ? getIcon()
            : Container(),
        Text(
          getDuration,
          style: widget.textStyle,
        ),
        widget.iconPosition == IconPosition.suffix && widget.isDiplayIcon
            ? getIcon()
            : Container(),
      ],
    );
  }

  Widget getIcon() {
    return Padding(
      padding: widget.iconPosition == IconPosition.prefix
          ? EdgeInsets.only(right: widget.iconPadding)
          : EdgeInsets.only(left: widget.iconPadding),
      child: iconWidget(),
    );
  }

  Icon? iconWidget() {
    if (widget.icon == null) {
      return Icon(
        Icons.timer,
        size: widget.iconSize,
        color: widget.iconColor,
      );
    }
    return widget.icon;
  }

  String get getDuration {
    int remaining = widget.controller.counter;
    final Duration remainingDuration = Duration(seconds: remaining);
    final Enum timerFormat = widget.timerFormat;
    late final String duration;

    if (timerFormat == TimerTextFormat.hh_mm_ss) {
      final hours = remainingDuration.inHours % 60;
      final minutes = remainingDuration.inMinutes % 60;
      final seconds = remaining % 60;

      final hoursString = '$hours'.padLeft(2, '0');
      final minutesString = '$minutes'.padLeft(2, '0');
      final secondsString = '$seconds'.padLeft(2, '0');

      duration = '$hoursString:$minutesString:$secondsString';
    } else if (timerFormat == TimerTextFormat.mm_ss) {
      final minutes = remainingDuration.inMinutes % 60;
      final seconds = remaining % 60;

      final minutesString = '$minutes'.padLeft(2, '0');
      final secondsString = '$seconds'.padLeft(2, '0');

      duration = '$minutesString:$secondsString';
    } else {
      final secondsString = '$remaining'.padLeft(2, '0');
      duration = secondsString;
    }

    return duration;
  }
}
