import 'dart:async';

import 'package:flexi_timer/flexi_timer.dart';
import 'package:flutter/material.dart';
import 'package:flutter_test/flutter_test.dart';

MaterialApp wrapWithMaterialApp(Widget child) {
  return MaterialApp(
    home: Scaffold(
      body: child,
    ),
  );
}

void main() {
  FlexiTimerController _controller = FlexiTimerController();
  Duration duration = const Duration(seconds: 5);
  testWidgets('FlexiTimer - no icon and no onComplete',
      (WidgetTester tester) async {
    await tester.runAsync(() async {
      await tester.pumpWidget(
        wrapWithMaterialApp(
          FlexiTimer(
            controller: _controller,
            duration: duration,
          ),
        ),
      );

      expect(find.byType(FlexiTimer), findsOneWidget);
      expect(find.byType(Icon), findsNothing);
      expect(find.byType(TextStyle), findsNothing);
    });
  });

  testWidgets('FlexiTimer - with icon', (WidgetTester tester) async {
    await tester.runAsync(() async {
      await tester.pumpWidget(
        wrapWithMaterialApp(
          FlexiTimer(
            controller: _controller,
            duration: duration,
            isDiplayIcon: true,
          ),
        ),
      );

      expect(find.byType(FlexiTimer), findsOneWidget);
      expect(find.byType(Icon), findsOneWidget);
      expect(find.byType(TextStyle), findsNothing);
    });
  });

  testWidgets('FlexiTimer - with onComplete', (WidgetTester tester) async {
    final onCompleteCompleter = Completer<bool>();
    await tester.pumpWidget(
      wrapWithMaterialApp(
        FlexiTimer(
          controller: _controller,
          duration: duration,
          onComplete: () {
            onCompleteCompleter.complete(true);
          },
        ),
      ),
    );
    await tester.pump(const Duration(seconds: 6));

    expect(find.byType(FlexiTimer), findsOneWidget);
    expect(find.byType(Icon), findsNothing);
    expect(onCompleteCompleter.isCompleted, isTrue);
  });
}
